
/* Copyright © 2012 Matthew Champion
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of mattunderscore.com nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MATTHEW CHAMPION BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

package com.mattunderscore.filter.contentnegotiation.parser;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.mattunderscore.http.headers.ContentTypeParser;
import com.mattunderscore.http.headers.LanguageParser;
import com.mattunderscore.http.headers.QContentType;
import com.mattunderscore.http.headers.QLanguage;
import com.mattunderscore.http.headers.UnParsableHeaderException;

/**
 * The variant element of the negotiation XML.
 * 
 * @author Matt Champion
 * @since 0.0.13
 */
public class Variant
{

    private static final int HASH_SEED = 11;
    private static final int HASH_MULT = 7;

    protected String urlPattern;
    protected String contentType;
    protected String language;
    protected Output output;

    public Variant()
    {
    }

    /**
     * Get the URL pattern of the variant.
     * @return The URL pattern of the variant
     * @since 0.0.13
     */
    public String getUrlPattern()
    {
        return urlPattern;
    }

    /**
     * Set the URL pattern of the variant.
     * @param value The URL pattern of the variant
     * @since 0.0.13
     */
    public void setUrlPattern(final String value)
    {
        this.urlPattern = value;
    }

    /**
     * Get the content type of the variant.
     * @return The content type of the variant
     * @since 0.0.13
     */
    public String getContentType()
    {
        return contentType;
    }

    /**
     * Get the structured content types of the variant.
     * @return The structured content types of the variant
     * @throws UnParsableHeaderException
     * @since 0.0.13
     */
    public List<QContentType> getQContentTypes()
            throws UnParsableHeaderException
    {
        ContentTypeParser parser = new ContentTypeParser();
        List<QContentType> types = new ArrayList<QContentType>();
        types.addAll(parser.parseAll(contentType));
        return types;
    }

    /**
     * Set the content type of the variant.
     * @param value The content type of the variant
     * @since 0.0.13
     */
    public void setContentType(final String value)
    {
        this.contentType = value;
    }

    /**
     * Get the language of the variant.
     * @return The language of the variant
     * @since 0.0.13
     */
    public String getLanguage()
    {
        return language;
    }

    /**
     * Get the structured languages of the variant.
     * @return The structured languages of the variant
     * @throws UnParsableHeaderException
     * @since 0.0.13
     */
    public List<QLanguage> getQLanguages() throws UnParsableHeaderException
    {
        LanguageParser parser = new LanguageParser();
        List<QLanguage> types = new ArrayList<QLanguage>();
        types.addAll(parser.parseAll(language));
        return types;
    }

    /**
     * Set the language of the variant.
     * @param value The language of the variant
     * @since 0.0.13
     */
    public void setLanguage(final String value)
    {
        this.language = value;
    }

    /**
     * Get the output of the variant.
     * @return The output of the variant
     * @since 0.0.13
     */
    public Output getOutput()
    {
        return output;
    }

    /**
     * Set the output of the variant.
     * @param value The output of the variant
     * @since 0.0.13
     */
    public void setOutput(final Output value)
    {
        this.output = value;
    }

    /**
     * Decide if the variant URL pattern matches the request path.
     * @param path Request path
     * @return Path matches pattern
     * @since 0.0.13
     */
    public boolean requestPathMatchesVariant(final String path)
    {
        Pattern pattern = Pattern.compile(urlPattern);
        Matcher m = pattern.matcher(path);
        return m.matches();
    }

    @Override
    public int hashCode()
    {
        int hash = HASH_SEED;
        hash = (hash * HASH_MULT);
        if (urlPattern != null)
        {
            hash += urlPattern.hashCode();
        }
        else
        {
            hash += 3;
        }
        hash = (hash * HASH_MULT);
        if (contentType != null)
        {
            hash += contentType.hashCode();
        }
        else
        {
            hash += 5;
        }
        hash = (hash * HASH_MULT);
        if (language != null)
        {
            hash += language.hashCode();
        }
        else
        {
            hash += 7;
        }
        hash = (hash * HASH_MULT);
        if (output != null)
        {
            hash += output.hashCode();
        }
        else
        {
            hash += 11;
        }
        return hash;
    }
    
    @Override
    public boolean equals(Object o)
    {
        if (o == null)
        {
            return false;
        }
        else if (o instanceof Variant)
        {
            Variant co = (Variant)o;
            boolean allEqual = true;
            if (this.urlPattern != null)
            {
                if (co.urlPattern != null)
                {
                    allEqual = allEqual && this.urlPattern.equals(co.urlPattern);
                }
                else
                {
                    return false;
                }
            }
            else if (co.urlPattern != null)
            {
                return false;
            }
            if (this.contentType != null)
            {
                if (co.contentType != null)
                {
                    allEqual = allEqual && this.contentType.equals(co.contentType);
                }
                else
                {
                    return false;
                }
            }
            else if (co.contentType != null)
            {
                return false;
            }
            if (this.language != null)
            {
                if (co.language != null)
                {
                    allEqual = allEqual && this.language.equals(co.language);
                }
                else
                {
                    return false;
                }
            }
            else if (co.language != null)
            {
                return false;
            }
            if (this.output != null)
            {
                if (co.output != null)
                {
                    allEqual = allEqual && this.output.equals(co.output);
                }
                else
                {
                    return false;
                }
            }
            else if (co.output != null)
            {
                return false;
            }
            return allEqual;
        }
        else
        {
            return false;
        }
    }
}
